/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

static void set_channel_single (char *single);

void
irc_socket (void)
{
  struct sockaddr_in sock;
  struct hostent *hp;
  int len;
  char buff[SIZE_BUFFER];
  pthread_t th;
  int flag;

  time_t t = time (NULL);

#ifdef HAVE_LIBSSL
  if (secure)
    {
      meth = SSLv23_client_method ();
      if (!(ctx = SSL_CTX_new (meth)))
	{
	  fprintf (stderr, "Error: ssl connection.\n");
	  return;
	}
    }
#endif

  if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
      fprintf (stderr, "Error: socket.\n");
      return;
    }

  flag = fcntl (fd, F_GETFL, 0);
  fcntl (fd, F_SETFL, flag | O_NONBLOCK);

  if (!(hp = gethostbyname (server)))
    {
      close (fd);
      fprintf (stderr, "Error: resolve %s.\n", server);
      return;
    }

  memset ((void *) &sock, 0, sizeof (sock));
  sock.sin_family = AF_INET;
  sock.sin_addr.s_addr = ((struct in_addr *) (hp->h_addr))->s_addr;
  sock.sin_port = htons (port);

  /* Ciclo per la connect: */
  while (1)
    {
      /* Se sono fuori tempo massimo: */
      if (time (NULL) - t >= 5)
	return;

      /* Connect con gestione degli errori: */
      if (connect (fd, (struct sockaddr *) &sock, sizeof (sock)) < 0)
	{
	  if (errno == ETIMEDOUT || errno == EINPROGRESS || errno == EALREADY
	      || errno == EAGAIN)
	    continue;


	  printf ("%d\n", errno);
	  close (fd);
	  fprintf (stderr, "Error: connect to: %s.\n", server);
	  return;
	}

      /* Ce l'ho fatta: */
      else
	break;
    }

  fcntl (fd, F_SETFL, flag);

#ifdef HAVE_LIBSSL
  if (secure)
    {
      if (!(ssl = SSL_new (ctx)))
	{
	  fprintf (stderr, "Error: ssl connection.\n");
	  return;
	}

      SSL_set_fd (ssl, fd);

      if (SSL_connect (ssl) <= 0)
	{
	  fprintf (stderr, "Error: ssl connection.\n");
	  return;
	}

    }
#endif

  set_nick ();

  set_quit = 0;

  if (pthread_create (&th, NULL, thread_xml, NULL))
    fatal ("Error: pthread.\n");

  while ((len = get_line (buff)))
    {

      if (buff[len - 1] == '\n')
	buff[len - 1] = 0;
      else
	buff[len] = 0;

      if (debug)
	fprintf (stderr, "%s\n", buff);

      if (!strncmp (buff, "PING ", 5))
	{
	  buff[1] = 'O';
	  irc_write (buff, len);
	}
      else
	{
	  char *cmd;
	  int flag = 0;

	  cmd = check_cmd (buff);

	  if (cmd)
	    {
	      switch (atoi (cmd))
		{
		case 409:	/* ERR_NOORIGIN */
		case 431:	/* ERR_NONICKNAMEGIVEN */
		case 432:	/* ERR_ERRONEUSNICKNAME */
		case 433:	/* ERR_NICKNAMEINUSE */
		case 436:	/* ERR_NICKCOLLISION */
		  set_nick ();
		  set_channel ();
		  flag = 1;
		  break;

		case 471:	/* ERR_CHANNELISFULL */
		case 474:	/* ERR_BANNEDFROMCHAN */
		  set_channel ();
		  flag = 1;
		  break;

		case 376:	/* END of MOTD */
		  set_identify ();
		  set_channel ();
		  flag = 1;
		  break;
		}

	      if (!flag)
		{
		  if (!strcmp (cmd, "KICK") && len > strlen (cmd))
		    {
		      char *who, *channel;

		      channel = check_part (cmd + strlen (cmd) + 1);
		      who =
			check_part (cmd + strlen (cmd) + 2 +
				    strlen (channel));

		      if (!strcmp (who, nick))
			set_channel_single (channel);

		      free (who);
		    }
		}
	    }
	}
    }

  set_quit = 1;
  pthread_join (th, NULL);

#ifdef HAVE_LIBSSL
  if (secure)
    SSL_shutdown (ssl);
#endif

  shutdown (fd, SHUT_RDWR);
  close (fd);

#ifdef HAVE_LIBSSL
  if (secure)
    {
      SSL_free (ssl);
      SSL_CTX_free (ctx);
    }
#endif
}

void
set_nick (void)
{
  char buff[SIZE_BUFFER];
  int len;

  len = snprintf (buff, SIZE_BUFFER, "NICK %s\n", nick);
  irc_write (buff, len);

  len =
    snprintf (buff, SIZE_BUFFER, "USER %s %s %s %s\n", nick, nick, nick,
	      nick);
  irc_write (buff, len);

  set_identify ();
}

void
set_identify (void)
{
  char buff[SIZE_BUFFER];
  int len;

  if (password)
    {
      len =
	snprintf (buff, SIZE_BUFFER, "PRIVMSG nickserv :identify %s\n",
		  password);
      irc_write (buff, len);
    }
}

void
set_channel (void)
{
  char buff[SIZE_BUFFER];
  struct channel *tmp;
  int len;

  tmp = channels;
  while (tmp)
    {

      len = snprintf (buff, SIZE_BUFFER, "PART %s\n", tmp->channel);
      irc_write (buff, len);

      len = snprintf (buff, SIZE_BUFFER, "JOIN %s\n", tmp->channel);
      irc_write (buff, len);

      tmp = tmp->next;
    }
}

static void
set_channel_single (char *single)
{
  char buff[SIZE_BUFFER];
  int len;

  len = snprintf (buff, SIZE_BUFFER, "PART %s\n", single);
  irc_write (buff, len);

  len = snprintf (buff, SIZE_BUFFER, "JOIN %s\n", single);
  irc_write (buff, len);

}

/* EOF */
